from django.contrib.contenttypes.models import ContentType
from drf_yasg.utils import swagger_serializer_method
from rest_framework import serializers

from dcim.api.nested_serializers import NestedPlatformSerializer
from netbox.api.fields import ContentTypeField
from netbox.api.serializers import NetBoxModelSerializer
from netbox.constants import NESTED_SERIALIZER_PREFIX
from netbox_storage.api.nested_serializers import NestedFilesystemSerializer, NestedDriveSerializer, \
    NestedMountedVolumeSerializer, NestedLinuxDeviceSerializer, NestedVolumeGroupSerializer
from netbox_storage.models import Drive, Filesystem, Partition, MountedVolume, StorageConfigurationDrive, \
    LinuxDevice, TemplateConfigurationDrive, PhysicalVolume, VolumeGroup, LogicalVolume
from utilities.api import get_serializer_for_model
from virtualization.api.nested_serializers import NestedClusterSerializer, NestedVirtualMachineSerializer


class FilesystemSerializer(serializers.ModelSerializer):
    class Meta:
        model = Filesystem
        fields = (
            "id",
            "filesystem",
            "description",
            "created",
            "last_updated",
            "custom_fields",
        )


class DriveSerializer(NetBoxModelSerializer):
    cluster = NestedClusterSerializer(required=False, allow_null=True)
    url = serializers.HyperlinkedIdentityField(view_name="plugins-api:netbox_storage-api:drive-detail")

    class Meta:
        model = Drive
        fields = (
            "id",
            "url",
            "display",
            "size",
            "cluster",
            "identifier",
            "description",
            "created",
            "last_updated",
            "custom_fields",
        )


class PartitionSerializer(serializers.ModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name='plugins-api:netbox_storage-api:partition-detail')
    drive = NestedDriveSerializer(required=False, allow_null=True)
    device = NestedLinuxDeviceSerializer(required=False, allow_null=True)

    class Meta:
        model = Partition
        fields = (
            "id",
            "url",
            "drive",
            "device",
            "size",
            "type",
            "description",
            "created",
            "last_updated",
            "custom_fields",
        )


class MountedVolumeSerializer(serializers.ModelSerializer):
    fs_type = NestedFilesystemSerializer(required=False, allow_null=True)

    class Meta:
        model = MountedVolume
        fields = (
            "id",
            "mount_point",
            "fs_type",
            "options",
            "description",
            "created",
            "last_updated",
            "custom_fields",
        )


class LinuxDeviceSerializer(serializers.ModelSerializer):
    content_type = ContentTypeField(
        queryset=ContentType.objects.all()
    )
    object = serializers.SerializerMethodField(read_only=True)

    class Meta:
        model = LinuxDevice
        fields = (
            "id",
            "device",
            "type",
            "size",
            'content_type', 'object_id', 'object',
            "created",
            "last_updated",
            "custom_fields",
        )

    @swagger_serializer_method(serializer_or_field=serializers.JSONField)
    def get_object(self, instance):
        serializer = get_serializer_for_model(instance.content_type.model_class(), prefix=NESTED_SERIALIZER_PREFIX)
        context = {'request': self.context['request']}
        return serializer(instance.object, context=context).data


class PhysicalVolumeSerializer(serializers.ModelSerializer):
    linux_device = NestedLinuxDeviceSerializer(required=False, allow_null=True)
    vg = NestedVolumeGroupSerializer(required=False, allow_null=True)

    class Meta:
        model = PhysicalVolume
        fields = (
            "id",
            "linux_device",
            "vg",
            "description",
            "created",
            "last_updated",
            "custom_fields",
        )


class VolumeGroupSerializer(serializers.ModelSerializer):

    class Meta:
        model = VolumeGroup
        fields = (
            "id",
            "vg_name",
            "description",
            "created",
            "last_updated",
            "custom_fields",
        )


class LogicalVolumeSerializer(serializers.ModelSerializer):
    vg = NestedVolumeGroupSerializer(required=False, allow_null=True)

    class Meta:
        model = LogicalVolume
        fields = (
            "id",
            "vg",
            "lv_name",
            "size",
            "description",
            "created",
            "last_updated",
            "custom_fields",
        )


class StorageConfigurationDriveSerializer(serializers.ModelSerializer):
    linux_volume = NestedMountedVolumeSerializer(required=False, allow_null=True)
    virtual_machine = NestedVirtualMachineSerializer(required=False, allow_null=True)

    class Meta:
        model = StorageConfigurationDrive
        fields = (
            "id",
            "device",
            "linux_volume",
            "created",
            "last_updated",
            "custom_fields",
        )


class TemplateConfigurationDriveSerializer(serializers.ModelSerializer):
    drive = NestedDriveSerializer(required=False, allow_null=True)
    platform = NestedPlatformSerializer(required=False, allow_null=True)

    class Meta:
        model = TemplateConfigurationDrive
        fields = (
            "id",
            "platform",
            "drive",
            "created",
            "last_updated",
            "custom_fields",
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['netbox_storage',
 'netbox_storage.api',
 'netbox_storage.filters',
 'netbox_storage.forms',
 'netbox_storage.migrations',
 'netbox_storage.tables',
 'netbox_storage.templatetags',
 'netbox_storage.views']

package_data = \
{'': ['*'],
 'netbox_storage': ['templates/netbox_storage/*',
                    'templates/netbox_storage/drive/*',
                    'templates/netbox_storage/inc/*',
                    'templates/netbox_storage/partition/*']}

setup_kwargs = {
    'name': 'netbox-storage',
    'version': '0.0.0.0.745',
    'description': 'Netbox Storage Plugin',
    'long_description': '# General\n## Build Project\nTo build the project go to log in the pypi web ui and get your token. Add your token to the local pypi config.\n```\npoetry config pypi-token.pypi pypi-XXXXX\n```\nAfter you made changes, change the version in the files pyproject.toml and netbox_storage/__init__.py\n\nNow you can build and publish the project.\n```\npoetry publish --build\n```\n\n## Use Project\nLink: https://github.com/netbox-community/netbox-docker/wiki/Using-Netbox-Plugins\n\ndocker-compose build --no-cache && docker-compose build --no-cache && docker-compose up -d\n\n\n## Directory structure\n\n```\n+- api - The API Classes, consitsts of Serializer, URL Mapper and Views\n+- filters - Filters of the models, the implementation of the method search, for searching\n+- forms - The ModelForm, ModelFilterForm, ModelImportForm, ModelBulkEditForm, the forms which will be displayed\n+- migrations - DB Django Migration steps\n+- tables - The ModelTable, which has the configuration on how the table looks like\n+- templates\n  +- netbox_storage - The detail view of each model\n    +- drive - The template content of drive, with base and partition model\n    +- inc - The template content box in the Virtual Machine Model\n    +- partition - The template content of partition, with base and physicalvolume model\n    +- physicalvolume - The template content of physicalvolume with base and linuxvolume model\n    +- volumegroup - The template content of volumegroup with base, logicalvolume and physicalvolume\n+- views - PhysicalvolumeListView, PhysicalvolumeView, PhysicalvolumeEditView, PhysicalvolumeDeleteView, \n           PhysicalvolumeBulkImportView, PhysicalvolumeBulkEditView, PhysicalvolumeBulkDeleteView\n```\n### Models\n#### ERM\n\n![The ERM of the Project](documents/erm.jpg?raw=true "ERM Diagram")\n\n#### Drive\nThe drive has 4 parameter:\n\n| Name           |           Example Value           |\n|:---------------|:---------------------------------:|\n| Virtualmachine | test-vm (Link zu virtual machine) |\n| Identifier     |           Festplatte 1            |\n| Cluster        |   STOR2000000 (Link zu cluster)   |\n| Size           |               50GB                |\n| System         |                No                 |\n\n#### Filesystem\nThe filesystem has 1 parameter:\n\n| Name | Example Value |\n|:-----|:-------------:|\n| fs   |     EXT4      |\n\n\n\ngit add . && git commit -m "0.0.0.0.272" && git push\n\n',
    'author': 'Tim Rhomberg',
    'author_email': 'timrhomberg@hotmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from rest_framework.decorators import action
from rest_framework.response import Response
from rest_framework.routers import APIRootView

from netbox.api.viewsets import NetBoxModelViewSet
from netbox_storage.api.serializers import (
    DriveSerializer,
    FilesystemSerializer, PartitionSerializer, MountedVolumeSerializer, LinuxDeviceSerializer, LogicalVolumeSerializer,
    PhysicalVolumeSerializer, VolumeGroupSerializer
)
from netbox_storage.filters import DriveFilter, FilesystemFilter, \
    PartitionFilter, MountedVolumeFilter, LinuxDeviceFilter, PhysicalVolumeFilter, VolumeGroupFilter, \
    LogicalVolumeFilter
from netbox_storage.models import Drive, Filesystem, Partition, MountedVolume, LinuxDevice, LogicalVolume, \
    PhysicalVolume, VolumeGroup


class NetboxStorageRootView(APIRootView):
    """
    NetboxDNS API root view
    """

    def get_view_name(self):
        return "NetboxStorage"


class DriveViewSet(NetBoxModelViewSet):
    queryset = Drive.objects.all()
    serializer_class = DriveSerializer
    filterset_class = DriveFilter

    @action(detail=True, methods=["get"])
    def drive(self, request, pk=None):
        drives = Drive.objects.filter(drive__id=pk)
        serializer = DriveSerializer(drives, many=True, context={"request": request})
        return Response(serializer.data)


class FilesystemViewSet(NetBoxModelViewSet):
    queryset = Filesystem.objects.all()
    serializer_class = FilesystemSerializer
    filterset_class = FilesystemFilter

    @action(detail=True, methods=["get"])
    def filesystem(self, request, pk=None):
        filesystem = Filesystem.objects.filter(filesystem__id=pk)
        serializer = FilesystemSerializer(filesystem, many=True, context={"request": request})
        return Response(serializer.data)


class PartitionViewSet(NetBoxModelViewSet):
    queryset = Partition.objects.all()
    serializer_class = PartitionSerializer
    filterset_class = PartitionFilter

    @action(detail=True, methods=["get"])
    def partition(self, request, pk=None):
        partition = Partition.objects.filter(partition__id=pk)
        serializer = PartitionSerializer(partition, many=True, context={"request": request})
        return Response(serializer.data)


class MountedVolumeViewSet(NetBoxModelViewSet):
    queryset = MountedVolume.objects.all()
    serializer_class = MountedVolumeSerializer
    filterset_class = MountedVolumeFilter

    @action(detail=True, methods=["get"])
    def mountedvolume(self, request, pk=None):
        mountedvolume = MountedVolume.objects.filter(mountedvolume__id=pk)
        serializer = MountedVolumeSerializer(mountedvolume, many=True, context={"request": request})
        return Response(serializer.data)


class LinuxDeviceViewSet(NetBoxModelViewSet):
    queryset = LinuxDevice.objects.all()
    serializer_class = LinuxDeviceSerializer
    filterset_class = LinuxDeviceFilter

    @action(detail=True, methods=["get"])
    def linuxdevice(self, request, pk=None):
        linuxdevice = LinuxDevice.objects.filter(linuxdevice__id=pk)
        serializer = LinuxDeviceSerializer(linuxdevice, many=True, context={"request": request})
        return Response(serializer.data)


class LogicalVolumeViewSet(NetBoxModelViewSet):
    queryset = LogicalVolume.objects.all()
    serializer_class = LogicalVolumeSerializer
    filterset_class = LogicalVolumeFilter

    @action(detail=True, methods=["get"])
    def logicalvolume(self, request, pk=None):
        logicalvolume = LogicalVolume.objects.filter(logicalvolume__id=pk)
        serializer = LogicalVolumeSerializer(logicalvolume, many=True, context={"request": request})
        return Response(serializer.data)


class PhysicalVolumeViewSet(NetBoxModelViewSet):
    queryset = PhysicalVolume.objects.all()
    serializer_class = PhysicalVolumeSerializer
    filterset_class = PhysicalVolumeFilter

    @action(detail=True, methods=["get"])
    def physicalvolume(self, request, pk=None):
        physicalvolume = PhysicalVolume.objects.filter(physicalvolume__id=pk)
        serializer = PhysicalVolumeSerializer(physicalvolume, many=True, context={"request": request})
        return Response(serializer.data)


class VolumeGroupViewSet(NetBoxModelViewSet):
    queryset = VolumeGroup.objects.all()
    serializer_class = VolumeGroupSerializer
    filterset_class = VolumeGroupFilter

    @action(detail=True, methods=["get"])
    def volumegroup(self, request, pk=None):
        volumegroup = VolumeGroup.objects.filter(volumegroup__id=pk)
        serializer = VolumeGroupSerializer(volumegroup, many=True, context={"request": request})
        return Response(serializer.data)
